package cn.wisenergy.service.excel.listener;

import cn.wisenergy.model.app.Resident;
import cn.wisenergy.service.app.ResidentService;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.metadata.Head;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Excel批量添加社区居民  监听器
 */
@Slf4j
public class ResidentListener extends AnalysisEventListener<Resident> {

    private List<Resident> list;

    private ResidentService residentService;

    private final int batch = 2000;

    private Map<Integer, String> headMap = new HashMap<>(); // 导入模板头内容
    private boolean isCorrectFormat = true; //结果标记 默认错误


    public ResidentListener(ResidentService residentService) {
        this.residentService = residentService;
        list = new ArrayList<>();
    }

    /**
     * 这个每一条数据解析都会来调用  ==
     *
     * @param resident
     * @param analysisContext
     */
    @Override
    public void invoke(Resident resident, AnalysisContext analysisContext) {
        if (!isCorrectFormat) { //格式是否正确
            return;
        }
        log.info("解析到一条居民数据>>>>>>>>>>：{}", resident);
        //此处做校验  本次没有校验
        if (resident != null) {
            resident.setIsDelete(0);
            resident.setCreateTime(new Date());
            resident.setUpdateTime(new Date());
            if (StringUtils.isEmpty(resident.getName())) {
                throw new RuntimeException("居民姓名不能为空！");
            }

            if (StringUtils.isEmpty(resident.getPhone())) {
                throw new RuntimeException("居民手机号不能为空！");
            } else {
                if (resident.getPhone().length() != 11) {
                    throw new RuntimeException("居民手机号错误！");
                }
            }

            if (null == resident.getSex()) {
                throw new RuntimeException("居民性别不能为空！");
            }

            if (StringUtils.isEmpty(resident.getCommunity())) {
                throw new RuntimeException("居民社区信息不能为空！");
            }

            if (StringUtils.isEmpty(resident.getStreet())) {
                throw new RuntimeException("居民街道信息不能为空！");
            }
            log.info("打印用户信息：" + resident.getPhone());
            Resident result = residentService.getByPhone(resident.getPhone());
            if (null != result) {
                throw new RuntimeException(resident.getName() + "的手机号" + resident.getPhone() + "重复");
            }
            list.add(resident);
        }

        //每500条插入一次数据库
        if (list.size() >= batch) {
            //判断数据库是否已有手机号的数据
            List<Resident> residents = this.removeExist(list);
            log.info("invoke批量插入居民数据，条数为>>>>>>>>>>：{}", residents.size());
            if (residents.size() > 0) {
                residentService.saveBatch(residents);
            }
            //保存后清除，防止大量数据导致内存耗尽
            list.clear();
        }
    }

    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param analysisContext
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        log.info("doAfterAllAnalysed解析玩一个居民sheet工作表>>>>>>>>>>");
        //如果集合不为空
        if (!CollectionUtils.isEmpty(list)) {
            //判断excel表中的手机号是否重复
            checkPhone(list);
            List<Resident> residents = this.removeExist(list);
            log.info("批量插入居民数据，条数为>>>>>>>>>>：{}", residents.size());
            if (residents.size() > 0) {
                residentService.saveBatch(residents);
            }
        }
    }

    /**
     * 校验导入模板表头是否正确
     *
     * @param headMap
     * @param context
     */
    public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
        //是否手动传入表头信息 没有则自动读取实体类头信息
        try {
            if (this.headMap == null || this.headMap.size() <= 0) {
                Map<Integer, Head> map = context.currentReadHolder().excelReadHeadProperty().getHeadMap();
                for (int i = 0; i < map.size(); i++) {
                    //封装成headMap
                    this.headMap.put(map.get(i).getColumnIndex(), map.get(i).getHeadNameList().get(0));//加入模板头
                }
            }
        } catch (Exception e) {
            isCorrectFormat = false; //导入错误
            throw new RuntimeException("导入失败，请联系管理员！");
        }

        if (this.headMap == null || headMap == null || this.headMap.size() != headMap.size()) {
            isCorrectFormat = false; //导入表头错误
            throw new RuntimeException("导入表的表头与模板表头不符!");
        } else {
            for (int i = 0; i < this.headMap.size(); i++) { //循环遍历判断表格头是否一一对应
                if (!this.headMap.get(i).equals(headMap.get(i))) {
                    isCorrectFormat = false; //导入表头错误
                    throw new RuntimeException("导入表的表头与模板表头不符!");
                }
            }
        }

    }

    /**
     * 数据库总已经存在的手机号不能重复添加
     *
     * @param data
     * @return
     */
    private List<Resident> removeExist(List<Resident> data) {
        //筛选出手机号
        Set<String> phones = data.stream().map(Resident::getPhone).collect(Collectors.toSet());
        if (CollectionUtils.isEmpty(phones)) {
            return data;
        }
        QueryWrapper<Resident> wrapper = new QueryWrapper<>();
        wrapper.in("phone", phones).select("phone");
        wrapper.eq("is_delete",0);
        List<Resident> listPhone = residentService.list(wrapper);
        Set<String> dbPhones = listPhone.stream().map(Resident::getPhone).collect(Collectors.toSet());
        //排除存在的数据
        List<Resident> noExist = data.stream().filter(d -> !dbPhones.contains(d.getPhone())).collect(Collectors.toList());
        return noExist;

    }

    private void checkPhone(List<Resident> list) {
        Set<String> set = new HashSet<>();
        for (Resident resident : list) {
            if (set.contains(resident.getPhone())) {
                throw new RuntimeException("导入EXCEL中手机号:" + resident.getPhone() + "重复！");
            } else {
                set.add(resident.getPhone());
            }
        }
    }

}
